/* Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.opengroup.osdu.notification.subscriptions;

import static org.junit.Assert.assertEquals;

import org.opengroup.osdu.core.common.http.json.HttpResponseBodyMapper;
import org.opengroup.osdu.core.common.legal.ILegalProvider;
import org.opengroup.osdu.core.common.legal.LegalAPIConfig;
import org.opengroup.osdu.core.common.legal.LegalFactory;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.legal.LegalException;
import org.opengroup.osdu.core.common.model.legal.LegalTag;
import org.opengroup.osdu.core.common.model.legal.Properties;
import com.amazonaws.services.sqs.model.Message;
import org.opengroup.osdu.notification.util.AwsTestUtils;
import org.opengroup.osdu.notification.util.Config;
import org.opengroup.osdu.notification.util.TestUtils;

import java.sql.Date;
import java.util.Collections;
import java.util.List;

public class LegalTagUtils {

    private final ILegalProvider legalService;

    public LegalTagUtils(DpsHeaders dpsHeaders, HttpResponseBodyMapper responseBodyMapper) {
        LegalAPIConfig legalConfig = LegalAPIConfig.builder().rootUrl(Config.Instance().LegalServicePath).build();
        LegalFactory legalFactory = new LegalFactory(legalConfig, responseBodyMapper);
        legalService = legalFactory.create(dpsHeaders);
    }

    public void addLegalTag(String name, String expDate) throws LegalException {
        LegalTag legalTag = createLegalTag(name, expDate);
        legalService.create(legalTag);
    }

    public void addLegalTag(String name) throws LegalException {
        addLegalTag(name, "2099-01-25");
    }

    public LegalMessage deleteLegalTag(String name) throws LegalException {
        legalService.delete(name);
        LegalMessageElement element = new LegalMessageElement(TestUtils.getOsduTenant(), name, "incompliant");
        return new LegalMessage(Collections.singletonList(element));
    }

    public void ensureDeleted(String name) {
        try {
            deleteLegalTag(name);
        } catch (LegalException e) {
            System.err.println(e.toString());
        }
    }

    public void assertFirstMessagesSimilar(LegalMessage expected, List<Message> receivedMessages) throws Exception {
        LegalMessage legalMessage = AwsTestUtils.unwrapSingleFirst(receivedMessages, LegalMessage.class);
        assertEquals(1, legalMessage.statusChangedTags().size());
        LegalMessageElement expectedElement = expected.statusChangedTags().get(0);
        LegalMessageElement actualElement = legalMessage.statusChangedTags().get(0);
        assertEquals(expectedElement.dataPartitionId(), actualElement.dataPartitionId());
        assertEquals(expectedElement.changedTagName(), actualElement.changedTagName());
        assertEquals(expectedElement.changedTagStatus(), actualElement.changedTagStatus());
    }

    private LegalTag createLegalTag(String name, String expDate) {
        LegalTag legalTag = new LegalTag();
        legalTag.setName(name);
        legalTag.setDescription(String.format("test for %s", name));
        Properties legalProps = new Properties();
        legalProps.setCountryOfOrigin(Collections.singletonList("US"));
        legalProps.setContractId("A1234");
        legalProps.setDataType("Public Domain Data");
        legalProps.setExpirationDate(Date.valueOf(expDate));
        legalProps.setOriginator("MyCompany");
        legalProps.setSecurityClassification("Public");
        legalProps.setExportClassification("EAR99");
        legalProps.setPersonalData("No Personal Data");
        legalTag.setProperties(legalProps);
        return legalTag;
    }
}

record LegalMessage(List<LegalMessageElement> statusChangedTags) {
}

record LegalMessageElement(String dataPartitionId, String changedTagName, String changedTagStatus) {
}
